'' BlinkPASM.spin
'' Object with methods that launch and communicate with a light blinking
'' process in another cog.

VAR

  long cog
  
  '    par      par + 4   par + 8
  long maskLed, stateLed, halfPeriod 

PUB start(ledPin, cycleTicks) : success
{{Start the blink process.
Parameters: ledPin is the pin number to blink and cycleTicks are the number of
clock ticks in one on/off cycle.}}

  maskLed := 1 << ledpin
  stateLed := 0
  setRate(cycleTicks)

  success := cog := 1+ cognew(@entry, @maskLed)

PUB stop
{{Stop the blink process and recover the cog for other uses.}}

  cogstop(cog -1)
  cog := 0

PUB getState : state
{{Get the state the state the process has the LED set to.  Returns 1 if on or 0
if off.}}

  state := stateLed

PUB setRate(cycleTicks)
{{Set the the blink rate by passing this funciton the number of click ticks in
an on/off cycle.}} 
  repeat while getState == 1
  halfPeriod := cycleTicks / 2  
  

DAT

              org     0                          ' Start at address 0 in cog ram

entry         mov     addr,       par            ' par = address of pinLed
              rdlong  ledmask,    addr           ' Get maskLed from Main RAM
              or      dira,       ledmask        ' OR w/ dira to make pin output

              add     addr,       #4             ' par + 4 = stateLed address
              mov     stateAddr,  addr           ' Store for later use

              add     addr,       #4             ' par + 8 = halfCycleTicks address
              mov     ticksAddr,  addr           ' Store for later use

              mov     t,          cnt            ' Mark current time
              rdlong  ticks,      ticksAddr      ' 1/2 cycle tics from app via Main RAM
              add     t,          ticks          ' Set first state change target

:loop         rdlong  ticks,      ticksAddr      ' 1/2 cycle tics from app via Main RAM
              waitcnt t,          ticks          ' Wait 1/2 cycle, set next target t

              xor     outa,       ledmask        ' Invert output state

              test    ledmask,    outa       wz  ' Check current output state
       if_nz  wrlong  isOn,       stateAddr      ' Communicate output state to app        
       if_z   wrlong  isOff,      stateAddr           

              jmp     #:loop                     ' Jump to :loop label

' Initialized data

isOn          long      1                        ' isOn stores 1
isOff         long      0                        ' isOff stores 0

' Uninitialized data
                                                               
addr          res      1                         ' Main RAM addresses 
ledmask       res      1                         ' Mask for LED control
stateAddr     res      1                         ' Address of LED state for app to monitor
ticksAddr     res      1                         ' Address where app stores half cycle ticks
ticks         res      1                         ' Local copy of half cycle ticks
t             res      1                         ' Target cnt ticks
       